/*
 * Copyright 2024 Luca Fulchir <luca.fulchir@runesauth.com>
 *
 * Licensed under the Apache License, Version 2.0 with LLVM exception (the
 * "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License and of the exception at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *     https://spdx.org/licenses/LLVM-exception.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

use ::std::{
    collections::HashMap,
    sync::{Arc, RwLock},
    vec::Vec,
};

type ArcLock<T> = Arc<RwLock<T>>;
type PkgMap = HashMap<crate::PkgName, ArcLock<dyn crate::Pkg>>;

/// Collection of packages, subset of what is present on one repository only
pub struct Collection {
    repos: RwLock<Vec<ArcLock<dyn crate::Repository>>>,
    pkgs: RwLock<PkgMap>,
    repos_pkgs: RwLock<HashMap<crate::RepoName, PkgMap>>,
    //repos_pkgs: RwLock<HashMap<crate::Repository, PkgMap>>,
}

impl Collection {
    pub fn new() -> Collection {
        const PKGNUM: usize = 512;
        Collection {
            repos: RwLock::new(Vec::with_capacity(8)),
            pkgs: RwLock::new(HashMap::with_capacity(PKGNUM)),
            repos_pkgs: RwLock::new(HashMap::with_capacity(PKGNUM)),
        }
    }
}
